/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.definition;
import java.util.ArrayList;
import java.util.List;

import javax.naming.ConfigurationException;

import de.bsvrz.kex.datex2.plugins.datensammlung.DatenSammlungsMethode;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungDynamisch;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungStatisch;
import de.bsvrz.kex.datex2.plugins.publizierung.PublizierungsMethode;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakte Klasse für Plugin-Definitionen (für den Export von Daten).
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public abstract class PluginExport extends KExD2Plugin {
  /** Intervall zwischen zwei dynamischen Publikationen. */
  private static final String PUBLIKATIONSPERIODE = "Publikationsperiode";

  /** Referenz auf Konvertierungsklasse für statische Daten. */
  protected DatexKonvertierungStatisch datexKonvertierungStatisch;
  
  /** Referenz auf Konvertierungsklasse für dynamische Daten. */
  protected DatexKonvertierungDynamisch datexKonvertierungDynamisch;
  
  /** Referenz auf Datensammlungsmethode. */
  protected DatenSammlungsMethode datenSammlungsMethode;
  
//  private String predefinedLocationVersion;
  
  /** Liste von Publikationsmethoden für statische Daten. */
  protected List<PublizierungsMethode> statischePublizierungsMethoden = new ArrayList<>();
  
  /** Liste von Publikationsmethoden für dynamische Daten. */
  protected List<PublizierungsMethode> dynamischePublizierungsMethoden = new ArrayList<>();

  /** Flag für erfolgreiche Initialisierung. */
  protected boolean isKomplettInitialisiert = false;
  
  /**
   * Konstruktor.
   * 
   * @param standardPeriod Standard-Aufrufintervall für dynamische Daten.
   */
  protected PluginExport(final int standardPeriod) {
    super(standardPeriod);
    instantiiereDatexKonvertierungStatisch();
//    if (this.datexKonvertierungStatisch == null) {
//      throw new IllegalStateException("Das Plugin hat keine Methode für die Konvertierung statischer Daten angegeben.");
//    }

    instantiiereDatexKonvertierungDynamisch();
//    if (this.datexKonvertierungDynamisch == null) {
//      throw new IllegalStateException("Das Plugin hat keine Methode für die Konvertierung dynamischer Daten angegeben.");
//    }
    
    instantiierePluginAusfuehrungsAlgorithmus();
    if (this.pluginAusfuehrungsAlgorithmus == null) {
      throw new IllegalStateException("Das Plugin hat keine Methode für die Ausführung des Plugins angegeben.");
    }

    instantiiereDatenSammlungsMethode();
    if (this.datenSammlungsMethode == null) {
      throw new IllegalStateException("Das Plugin hat keine Methode für die Datensammlung angegeben.");
    }
    
    instantiiereStatischePublizierungsMethode();
    instantiiereDynamischePublizierungsMethode();  
  }
  
  /**
   * Abstrakte Methode / Erzwingt Instantiierung der dynamischen Publikationmethoden.
   */
  public abstract void instantiiereDynamischePublizierungsMethode();

   /**
    * Abstrakte Methode / Erzwingt Instantiierung der statischen Publikationmethoden.
    */
  public abstract void instantiiereStatischePublizierungsMethode();

  /**
   * Abstrakte Methode / Erzwingt Instantiierung der statische Konvertierungsmethode.
   */
  public abstract void instantiiereDatexKonvertierungStatisch();

  /**
   * Abstrakte Methode / Erzwingt Instantiierung der dynamischen Konvertierungsmethode.
   */
  public abstract void instantiiereDatexKonvertierungDynamisch();
  
  /**
   * Abstrakte Methode / Erzwingt Instantiierung des Plugin-Ausführungsalgorithms.
   */
  public abstract void instantiierePluginAusfuehrungsAlgorithmus();

  /**
   * Abstrakte Methode / Erzwingt Instantiierung der Datensammlungsmethode.
   */
  public abstract void instantiiereDatenSammlungsMethode();
  
  /**
   * Rückgabe der statischen Konvertierungsklassen-Instanz.
   * 
   * @return Konvertierungsklasse.
   */

  public DatexKonvertierungStatisch getDatexKonvertierungStatisch() {
    return datexKonvertierungStatisch;
  }
  
  /**
   * Rückgabe der dynamischen Konvertierungsklassen-Instanz.
   * 
   * @return Konvertierungsklasse.
   */

  public DatexKonvertierungDynamisch getDatexKonvertierungDynamisch() {
    return datexKonvertierungDynamisch;
  }
  
  /**
   * Rückgabe der Publizierungsmethoden für statische Daten.
   * 
   * @return Liste mit Publizierungsmethodenklassen-Instanzen
   */

  public List<PublizierungsMethode> getStatischePublizierungsMethoden() {
    return statischePublizierungsMethoden;
  }
    
  /**
   * Rückgabe der Publizierungsmethoden für dynamische Daten.
   * 
   * @return Liste mit Publizierungsmethodenklassenn-Instanzen
   */

  public List<PublizierungsMethode> getDynamischePublizierungsMethoden() {
    return dynamischePublizierungsMethoden;
  }
  
  /**
   * Übergangsmethode. Deprecated.
   * 
   * @return x
   */
  public String getPredefinedLocationVersion() {
    String predefinedLocationVersion = "ha";
    return predefinedLocationVersion;
  }
  
  /**
   * Rückgabe der Datensammlungsmethodenklasseninstanz.
   * 
   * @return Instanz DatensammlungsMethode
   */
  public DatenSammlungsMethode getDatenSammlungsMethode() {
    return datenSammlungsMethode;
  }
  
  /** 
   * Initialisiere alle assoziierten Objekte.
   * 
   * @throws ConfigurationException Fehler bei der Initialisierung
   */

  public void initialisiereAbhaengigkeiten() throws ConfigurationException {
    if (!datenSammlungsMethode.initialisiere(objekte)) {
      throw new ConfigurationException("In Kontext '" + getKonfigurationsModul().getContext() 
          + "' : DatenSammlungsMethode konnte nicht initialisiert werden.");
    }
    if (datexKonvertierungStatisch != null) {
      datexKonvertierungStatisch.initialisiere(objekte);
    }

    if (datexKonvertierungDynamisch != null) {
      datexKonvertierungDynamisch.initialisiere(objekte);
      
//    ) {
//        throw new ConfigurationException("In Kontext '" + getKonfigurationsModul().getContext() 
//            + "' : Datex-Konvertierung (dynamisch) konnte nicht initialisiert werden.");      
//      }
    }
    if (!pluginAusfuehrungsAlgorithmus.initialisiere()) {
      throw new ConfigurationException("In Kontext '" + getKonfigurationsModul().getContext() 
          + "' : ExecutionAlgorithm konnte nicht initialisiert werden.");      
    }
    for (PublizierungsMethode methode : dynamischePublizierungsMethoden) {
      if (!methode.initialisiere()) {
        throw new ConfigurationException("In Kontext '" + getKonfigurationsModul().getContext() 
            + "' : Publizierung nach " + methode.getPublizierungsMethodenName() 
            + " (dynamisch) konnte nicht initialisiert werden.");      
      }
    }
    for (PublizierungsMethode method : statischePublizierungsMethoden) {
      if (!method.initialisiere()) {
        throw new ConfigurationException("In Kontext '" + getKonfigurationsModul().getContext() 
            + "' : Publizierung nach " + method.getPublizierungsMethodenName() 
            + " (statisch) konnte nicht initialisiert werden.");      
      }
    }
    
    if (getKonfigurationsModul().hasData(PUBLIKATIONSPERIODE)) {
      Integer newPeriod = getKonfigurationsModul().getInteger(PUBLIKATIONSPERIODE);
      if (newPeriod != null) {
        this.period = newPeriod;
      } else {
        Debug.getLogger().warning("In Kontext '" + getKonfigurationsModul().getContext() 
            + ": Fehler beim Lesen von " + PUBLIKATIONSPERIODE);
      }
    }
    
    isKomplettInitialisiert = true;
  }
  
  @Override
  public boolean isInitialisiert() {  
	  return isKomplettInitialisiert;
	}
}
