/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.DatexKonvertierungStatisch;
import de.bsvrz.sys.funclib.debug.Debug;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.ConfidentialityValueEnum;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.CountryEnum;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.D2LogicalModel;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.Exchange;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.HeaderInformation;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.InformationStatusEnum;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.InternationalIdentifier;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.Point;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.PointByCoordinates;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.PointCoordinates;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsRecord;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsTablePublication;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsTextDisplayCharacteristics;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsTypeEnum;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsUnitRecord;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsUnitRecordVmsIndexVmsRecord;
import eu.datex2.schema.statisch.vms.table.anzeigetafeln.VmsUnitTable;

/**
 * Klasse zur Konvertierung statischer Daten (Anzeigequerschnitte).
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 *
 */

public class DWiStaVmsTable implements DatexKonvertierungStatisch {
  /** Referenz auf Plugin-Definitions-Klasse. */
  private PluginExport plugin;

  /** Schlüssel NationalIdentifier. */
  private static final String KEY_NATIONAL_IDENTIFIER = "National-Identifier";

  /** Schlüssel VmsTableId. */
  private static final String KEY_VMS_TABLE_ID = "VmsTableId";

  /** Schlüssel VmsTableVersion. */
  private static final String KEY_VMS_TABLE_VERSION = "VmsTableVersion";

  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();

  /** Id der VmsTable. */
  private String vmsTableId;

  /** NationalIdentifier. */
  private String nationalIdentifier;

  /** Präfix für Log-Ausgaben. */
  private String logPrefix = "";

  /** Standard-Ausgabe für Versionsstring. */
  private static final String STANDARD_VERSION_STRING = "unknown";

  /** Version der PredefinedLocation. */
  private String version = STANDARD_VERSION_STRING;

  /** Akzeptierte Datenverteiler-Typen. */
  private static final String TYP_QUERSCHNITT = "anzeigeQuerschnitt";

  /** Akzeptierter Datenverteiler-Typ: Anzeige. */
  private static final String TYP_ANZEIGE = "anzeige";

  /** Zuordnung Zeichen der AQ => ASCII-Zeichen zur Ausgabe. */
  private Map<String, String> zeichen = new HashMap<String, String>();

  /** Liste der Anzeigetafeln. */
  private List<AnzeigeTafel> anzeigetafelListe = new ArrayList<AnzeigeTafel>();

  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitionsinstanz.
   */
  public DWiStaVmsTable(final PluginExport plugin) {
    this.plugin = plugin;
  }

  /**
   * Aktuell verwendete Version der VmsTable.
   * 
   * @return Versionsstring
   */
  public String getVmsTableVersion() {
    return version;
  }

  @Override
  public Object getDatexII(final Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options) {
    boolean versionFree = WerkzeugSammlung.isVersionFree(options);

    String newVersion = WerkzeugSammlung.erzeugeVersionsString(versionFree);

    zeichen = getZeichenMap(plugin);

    D2LogicalModel d2l = new D2LogicalModel();

    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);
    VmsTablePublication payload = new VmsTablePublication();
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
    payload.setHeaderInformation(headerInformation);

    payload.setPublicationTime(WerkzeugSammlung.erzeugeVersionsdatum(versionFree));

    List<SystemObject> anzeigeQuerschnitteAuswertung = getAnzeigeQuerschnitteAuswertung();
    List<SystemObject> anzeigeObjekte = new ArrayList<>();
    List<SystemObject> anzeigeQuerschnitte = new ArrayList<>();

    for (ObjekteAtgAsp objekteAtgAsp : objekteAtgAspListe) {
      for (SystemObject mq : objekteAtgAsp.getSystemObjekte()) {
        if (objekteAtgAsp.getTyp().getName().equalsIgnoreCase(TYP_ANZEIGE)) {
          anzeigeObjekte.add(mq);
        } else if (objekteAtgAsp.getTyp().getName().equalsIgnoreCase(TYP_QUERSCHNITT)) {
          anzeigeQuerschnitte.add(mq);
        }
      }
    }

    // LOGGER.info("1. anzeigeQuerschnitte.size() " + anzeigeQuerschnitte.size());
    VmsUnitTable vmsUnitTable = getVmsUnits(anzeigeQuerschnitteAuswertung, anzeigeQuerschnitte, newVersion);
    // LOGGER.info("2. anzeigeQuerschnitte.size() " + anzeigeQuerschnitte.size());
    addAndereVmsUnits(vmsUnitTable, anzeigeQuerschnitte, anzeigeObjekte, newVersion);

    if (vmsUnitTable.getVmsUnitRecords().size() < 1) {
      return null;
    } else {
      if (!versionFree) {
        this.version = newVersion;
        try {
          plugin.getPersistenzModul().putData(KEY_VMS_TABLE_VERSION, version);
        } catch (Exception e) {
          LOGGER.warning(logPrefix + "VmsTableVersion kann nicht persistiert werden.", e);
        }
      }
      payload.getVmsUnitTables().add(vmsUnitTable);
      d2l.setPayloadPublication(payload);
      return d2l;
    }
  }

  /**
   * Generiert VMSUnits aus einer Liste von AnzeigeQuerschnitten, die nicht im
   * Datenverteiler mit einer Anzeige verknüpft sind, ordnet ihr Anzeigen zu und
   * fügt sie einer VmsUnitTable hinzu.
   * 
   * @param vmsUnitTable VmsUnitTable
   * @param anzeigeQuerschnitte List(SystemObject) AnzeigeQuerschnitte
   * @param anzeigeObjekte List(SystemObject) AnzeigeObjekte
   * @param newVersion Version der VmsTable
   */

  protected void addAndereVmsUnits(final VmsUnitTable vmsUnitTable, final List<SystemObject> anzeigeQuerschnitte,
      final List<SystemObject> anzeigeObjekte, final String newVersion) {
    // LOGGER.info("getAndereVmsUnits()");
    // LOGGER.info("anzeigeQuerschnitte.size() " + anzeigeQuerschnitte.size());
    for (SystemObject aQuerschnitte : anzeigeQuerschnitte) {
      AttributeGroup attGruppe = aQuerschnitte.getDataModel().getAttributeGroup("atg.punktKoordinaten");
      Data data = aQuerschnitte.getConfigurationData(attGruppe);
      PointByCoordinates pointByCoordinates = new PointByCoordinates();
      PointCoordinates pointCoordinates = new PointCoordinates();
      if (data != null) {
        pointCoordinates = setKoordinaten(data);
      } else {
        pointCoordinates.setLatitude(0);
        pointCoordinates.setLongitude(0);
      }
      Point point = new Point(); // extends NetworkLocation -> extends Location
      pointByCoordinates.setPointCoordinates(pointCoordinates);
      point.setPointByCoordinates(pointByCoordinates);
      int numberOfVms = 0;
      VmsUnitRecord vmsUnitRecord = new VmsUnitRecord();
      vmsUnitRecord.setVersion(newVersion);
      String pid = aQuerschnitte.getPid();

      vmsUnitRecord.setId(pid);
      for (SystemObject anzeigen : anzeigeObjekte) {
        if (anzeigen.getPid().trim().toLowerCase().startsWith(pid.trim().toLowerCase())) {
          VmsRecord vmsRecord = new VmsRecord();
          VmsUnitRecordVmsIndexVmsRecord vmsUnitRecordVmsIndexVmsRecord = new VmsUnitRecordVmsIndexVmsRecord();
          VmsTextDisplayCharacteristics vmsTextDisplayCharacteristics = new VmsTextDisplayCharacteristics();
          vmsUnitRecordVmsIndexVmsRecord.setVmsIndex(numberOfVms);
          vmsRecord.setVmsLocation(point);
          // TODO Woher diese beiden Werte nehmen?
          vmsRecord.setVmsType(VmsTypeEnum.OTHER);
          vmsTextDisplayCharacteristics.setTextLanternsPresent(false);
          vmsRecord.setVmsTextDisplayCharacteristics(vmsTextDisplayCharacteristics);
          vmsUnitRecordVmsIndexVmsRecord.setVmsRecord(vmsRecord);
          vmsUnitRecord.getVmsRecords().add(vmsUnitRecordVmsIndexVmsRecord);
          numberOfVms++;
        }
      }
      if (numberOfVms > 0) {
        vmsUnitRecord.setNumberOfVms(BigInteger.valueOf(numberOfVms));
        vmsUnitTable.getVmsUnitRecords().add(vmsUnitRecord);
      }
    }
  }

  /**
   * Generiert VMSUnits aus einer Liste von anzeigeQuerschnitteAuswertung und
   * verküpft damit Anzeigen mit AnzeigeQuerschnitten ordnet ihr Anzeigen zu und
   * fügt sie einer VmsUnitTable hinzu.
   * 
   * @param anzeigeQuerschnitteAuswertung List(SystemObject)
   *          anzeigeQuerschnitteAuswertung
   * @param anzeigeQuerschnitte List(SystemObject) AnzeigeQuerschnitte
   * @param newVersion Version der VmsUnit Tabelle
   * @return VmsUnitTable
   */

  protected VmsUnitTable getVmsUnits(final List<SystemObject> anzeigeQuerschnitteAuswertung,
      final List<SystemObject> anzeigeQuerschnitte, final String newVersion) {
    // LOGGER.info("getVmsUnits()");
    VmsUnitTable vmsUnitTable = new VmsUnitTable();
    vmsUnitTable.setId(vmsTableId);
    vmsUnitTable.setVersion(newVersion);

    for (SystemObject aqAuswertung : anzeigeQuerschnitteAuswertung) {
      AttributeGroup attributGruppe = aqAuswertung.getDataModel().getAttributeGroup("atg.anzeigeQuerschnittAuswertung");
      Data d = aqAuswertung.getConfigurationData(attributGruppe);
      SystemObject anzeigeQuerschnittObj = d.getReferenceValue("AnzeigeQuerschnitt").getSystemObject();
      anzeigeQuerschnitte.remove(anzeigeQuerschnittObj);
      AttributeGroup attGruppe = anzeigeQuerschnittObj.getDataModel().getAttributeGroup("atg.punktKoordinaten");
      Data data = anzeigeQuerschnittObj.getConfigurationData(attGruppe);
      PointByCoordinates pointByCoordinates = new PointByCoordinates();
      PointCoordinates pointCoordinates = new PointCoordinates();
      if (data != null) {
        pointCoordinates = setKoordinaten(data);
      } else {
        pointCoordinates.setLatitude(0);
        pointCoordinates.setLongitude(0);
      }
      int numberOfVms = 0;
      VmsUnitRecord vmsUnitRecord = new VmsUnitRecord();
      vmsUnitRecord.setVersion(newVersion);
      String pid = anzeigeQuerschnittObj.getPid();
      vmsUnitRecord.setId(pid);
      Point point = new Point(); // extends NetworkLocation -> extends Location
      pointByCoordinates.setPointCoordinates(pointCoordinates);
      point.setPointByCoordinates(pointByCoordinates);
      AttributeGroup aGr = aqAuswertung.getDataModel().getAttributeGroup("atg.anzeigeQuerschnittAuswertungSpalten");
      Data dat = aqAuswertung.getConfigurationData(aGr);
      boolean hasLeft = false;
      boolean hasRight = false;
      boolean hasMiddle = false;
      // LOGGER.info("dat.toString() " + dat.toString());
      Array anzeigenArr = dat.getArray("Anzeigen");
      List<AnzeigeZeile> anzeigeLinksList = new ArrayList<AnzeigeZeile>();
      List<AnzeigeZeile> anzeigeRechtsList = new ArrayList<AnzeigeZeile>();
      List<AnzeigeZeile> anzeigeMitteList = new ArrayList<AnzeigeZeile>();

      for (int i = 0; i < anzeigenArr.getLength(); i++) {
        Data da = anzeigenArr.getItem(i);
        String spaltenbezeichnung = da.getTextValue("Spaltenbezeichnung").getText().toLowerCase();
        int spaltennummer = (int) da.getUnscaledValue("Spaltennummer").doubleValue();
        SystemObject anzeigeO = da.getReferenceValue("Anzeige").getSystemObject();
        String anzeigePid = anzeigeO.getPid();
        AnzeigeZeile anzeigeZeile = new AnzeigeZeile(anzeigePid, spaltennummer);
        if (spaltenbezeichnung.endsWith("rechts")) {
          anzeigeRechtsList.add(anzeigeZeile);
          hasRight = true;
        } else if (spaltenbezeichnung.endsWith("links")) {
          anzeigeLinksList.add(anzeigeZeile);
          hasLeft = true;
        } else if (spaltenbezeichnung.endsWith("mitte")) {
          hasMiddle = true;
          anzeigeMitteList.add(anzeigeZeile);
        }
      }

      if (hasLeft) {
        VmsRecord vmsRecord = new VmsRecord();
        VmsUnitRecordVmsIndexVmsRecord vmsUnitRecordVmsIndexVmsRecord = new VmsUnitRecordVmsIndexVmsRecord();
        VmsTextDisplayCharacteristics vmsTextDisplayCharacteristics = new VmsTextDisplayCharacteristics();
        vmsUnitRecordVmsIndexVmsRecord.setVmsIndex(numberOfVms);
        vmsRecord.setVmsLocation(point);
        // TODO Woher diese beiden Werte nehmen?
        vmsRecord.setVmsType(VmsTypeEnum.OTHER);
        vmsTextDisplayCharacteristics.setTextLanternsPresent(false);
        vmsRecord.setVmsTextDisplayCharacteristics(vmsTextDisplayCharacteristics);
        vmsUnitRecordVmsIndexVmsRecord.setVmsRecord(vmsRecord);
        vmsUnitRecord.getVmsRecords().add(vmsUnitRecordVmsIndexVmsRecord);
        numberOfVms++;
      } else {
        anzeigeLinksList = null;
      }

      if (hasRight) {

        VmsRecord vmsRecord = new VmsRecord();
        VmsUnitRecordVmsIndexVmsRecord vmsUnitRecordVmsIndexVmsRecord = new VmsUnitRecordVmsIndexVmsRecord();
        VmsTextDisplayCharacteristics vmsTextDisplayCharacteristics = new VmsTextDisplayCharacteristics();
        vmsUnitRecordVmsIndexVmsRecord.setVmsIndex(numberOfVms);
        vmsRecord.setVmsLocation(point);
        // TODO Woher diese beiden Werte nehmen?
        vmsRecord.setVmsType(VmsTypeEnum.OTHER);
        vmsTextDisplayCharacteristics.setTextLanternsPresent(false);
        vmsRecord.setVmsTextDisplayCharacteristics(vmsTextDisplayCharacteristics);
        vmsUnitRecordVmsIndexVmsRecord.setVmsRecord(vmsRecord);
        vmsUnitRecord.getVmsRecords().add(vmsUnitRecordVmsIndexVmsRecord);
        numberOfVms++;
      } else {
        anzeigeRechtsList = null;
      }

      if (hasMiddle) {

        VmsRecord vmsRecord = new VmsRecord();
        VmsUnitRecordVmsIndexVmsRecord vmsUnitRecordVmsIndexVmsRecord = new VmsUnitRecordVmsIndexVmsRecord();
        VmsTextDisplayCharacteristics vmsTextDisplayCharacteristics = new VmsTextDisplayCharacteristics();
        vmsUnitRecordVmsIndexVmsRecord.setVmsIndex(numberOfVms);
        vmsRecord.setVmsLocation(point);
        // TODO Woher diese beiden Werte nehmen?
        vmsRecord.setVmsType(VmsTypeEnum.OTHER);
        vmsTextDisplayCharacteristics.setTextLanternsPresent(false);
        vmsRecord.setVmsTextDisplayCharacteristics(vmsTextDisplayCharacteristics);
        vmsUnitRecordVmsIndexVmsRecord.setVmsRecord(vmsRecord);
        vmsUnitRecord.getVmsRecords().add(vmsUnitRecordVmsIndexVmsRecord);
        numberOfVms++;
      } else {
        anzeigeMitteList = null;
      }

      AnzeigeTafel anzeigeTafel = new AnzeigeTafel(pid, anzeigeLinksList, anzeigeMitteList, anzeigeRechtsList);
      anzeigetafelListe.add(anzeigeTafel);
      vmsUnitRecord.setNumberOfVms(BigInteger.valueOf(numberOfVms));
      vmsUnitTable.getVmsUnitRecords().add(vmsUnitRecord);
    }

    return vmsUnitTable;
  }

  /**
   * Holt eine Liste von anzeigeQuerschnitteAuswertung-Objekten aus dem DaV.
   * 
   * @return List (SystemObject)
   */

  protected List<SystemObject> getAnzeigeQuerschnitteAuswertung() {
    ClientDavInterface davConnection = plugin.getDaVConnection();
    ObjekteAtgAsp objekteAtgAsp = new ObjekteAtgAsp(davConnection, "typ.anzeigeQuerschnittAuswertung",
        "atg.anzeigeQuerschnittAuswertung", "asp.eigenschaften");
    List<SystemObject> sol = objekteAtgAsp.getSystemObjekte();
    return sol;
  }

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    String con = "eu.datex2.schema.statisch.vms.table.anzeigetafeln";
    JAXBContext jbc = JAXBContext.newInstance(con);
    return jbc;
  }

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) {
    // if (!WerkzeugSammlung.initialize()) {
    // return false;
    // }

    logPrefix = "In Kontext '" + plugin.getKonfigurationsModul().getContext() + "': ";

    if (!plugin.getKonfigurationsModul().hasData(KEY_VMS_TABLE_ID)) {
      LOGGER.error(logPrefix + "Das Feld '" + KEY_VMS_TABLE_ID + "' wurde nicht in der Konfiguration angegeben.");
      // return false;
    } else {
      vmsTableId = plugin.getKonfigurationsModul().getString(KEY_VMS_TABLE_ID);
    }

    if (!plugin.getKonfigurationsModul().hasData(KEY_NATIONAL_IDENTIFIER)) {
      LOGGER
          .error(logPrefix + "Das Feld '" + KEY_NATIONAL_IDENTIFIER + "' wurde nicht in der Konfiguration angegeben.");
      // return false;
    } else {
      nationalIdentifier = plugin.getKonfigurationsModul().getString(KEY_NATIONAL_IDENTIFIER);
    }

    if (plugin.getPersistenzModul().hasData(KEY_VMS_TABLE_VERSION)) {
      this.version = plugin.getPersistenzModul().getString(KEY_VMS_TABLE_VERSION);
    }

  }

  /**
   * Setzt Koordinaten aus Data in ein PointCoordinates Objekt.
   * 
   * @param d Data
   * @return PointCoordinates
   */

  private static PointCoordinates setKoordinaten(final Data d) {
    PointCoordinates pointCoordinates = new PointCoordinates();
    String posLat = d.getScaledValue("y").getValueText();
    String posLong = d.getScaledValue("x").getValueText();
    float fLat = 0;
    float fLong = 0;
    try {
      fLat = Float.parseFloat(posLat.replace(",", "."));
      fLong = Float.parseFloat(posLong.replace(",", "."));
    } catch (Exception e) {
      LOGGER.error(e.getMessage());
    }
    pointCoordinates.setLatitude(fLat);
    pointCoordinates.setLongitude(fLong);
    return pointCoordinates;
  }

  /**
   * Baut eine Hashmap mit allen Zeichen vom typ.zeichen mit den PISs als key und
   * dem Zeichenwert als Values.
   * 
   * @param plugin PluginExport
   * @return Map (String, String)
   */

  protected Map<String, String> getZeichenMap(final PluginExport plugin) {
    Map<String, String> zeichenMap = new HashMap<String, String>();
    ClientDavInterface davConnection = plugin.getDaVConnection();
    ObjekteAtgAsp objekteAtgAsp = new ObjekteAtgAsp(davConnection, "typ.zeichen", "atg.zeichen", "asp.eigenschaften");
    List<SystemObject> sol = objekteAtgAsp.getSystemObjekte();
    for (SystemObject sysObj : sol) {
      AttributeGroup aGr = sysObj.getDataModel().getAttributeGroup("atg.zeichen");
      Data dat = sysObj.getConfigurationData(aGr);
      String buchstabenZeichen = dat.getTextValue("Zeichenname").getText();
      String buchstabe = getBuchstabe(buchstabenZeichen);
      // LOGGER.info("sysObj.getPid(), buchstabe: " + sysObj.getPid() + " " +
      // buchstabe);
      zeichenMap.put(sysObj.getPid(), buchstabe);
    }
    return zeichenMap;

  }

  /**
   * wandelt einen String aus dem DaV (typ.zeichen-&gt;zeichenname) in einen
   * Buchstaben.
   * 
   * @param sin String
   * @return String
   */

  protected String getBuchstabe(final String sin) {
    String b = " ";
    String s = sin.replace("MitOberUndUnterstrich", "");
    switch (s) {
    case "AnschlussstelleAuf":
    case "AnschlussstelleZu":
    case "Zwischenraum":
    case "zwischenraum":
      b = " ";
      break;
    case "BAB1StelligZu":
    case "BAB2StelligZu":
    case "BAB3StelligZu":
      b = "";
      break;
    case "BAB1StelligAuf":
    case "BAB2StelligAuf":
    case "BAB3StelligAuf":
      b = "A";
      break;
    case "PfeilAuf":
      b = "->";
      break;
    default:
      b = s;
      break;
    }

    return b;
  }

  /**
   * Rückgabe der Zuordnung AQ-Zeichen =&gt; ASCII-Zeichen.
   *
   * @return Map (String,String)
   */
  public Map<String, String> getZeichen() {
    return this.zeichen;
  }

  /**
   * Rückgabe der Liste der Anzeigetafeln.
   * 
   * @return List (AnzeigeTafel)
   */

  public List<AnzeigeTafel> getAnzeigetafelListe() {
    return this.anzeigetafelListe;
  }

}
