/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import eu.datex2.schema.dynamisch.baustellen.GroupOfLocations;
import eu.datex2.schema.dynamisch.baustellen.Linear;

/**
 * Wrapper f&uuml;r {@link GroupOfLocations}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public class Datex2Ortsreferenzen {

	private GroupOfLocations groupOfLocations;
	private Data data;

	/**
	 * Erzeugt ein neues Objekt.
	 * 
	 * @param data
	 *            zugeordnete DAV-Daten
	 * @throws Datex2BaustelleException
	 *             bei Fehlern
	 */
	public Datex2Ortsreferenzen(final Data data) throws Datex2BaustelleException {
		this.data = data;

		if (isLinear()) {
			groupOfLocations = new Datex2OrtsreferenzenLinear(data);
		} else if (isPunkt()) {
			groupOfLocations = new Datex2OrtsreferenzenPoint(data);
		} else {
			throw new Datex2BaustelleException(
					"Es sind keine Ortsreferenzen versorgt oder der Typ der Ortsreferenz (linear oder Punkt) kann nicht bestimmt werden!");
		}
	}

	/**
	 * Erzeugt ein neues Objekt aus {@link GroupOfLocations}.
	 * 
	 * @param groupOfLocations
	 *            {@link GroupOfLocations}
	 */
	public Datex2Ortsreferenzen(final GroupOfLocations groupOfLocations) {
		this.groupOfLocations = groupOfLocations;
	}

	/**
	 * Pr&uuml;ft, ob es sich um eine lineare Ortsreferenz handelt.
	 * 
	 * @return
	 */
	private boolean isLinear() {
		return !isPunkt()
				&& ((data.getArray("OrtsRefAlertC").getLength() > 0) || (data.getArray("OrtsRefAsb").getLength() > 0));
	}

	/**
	 * Pr&uuml;ft, ob es sich um eine punktuelle Ortsreferenz handelt. <br>
	 * <p>
	 * AlertC: (BisLCLPunkt == 0) || (BisLCLPunkt==VonLCLPunkt &&
	 * BisLCLOffset==VonLCLOffset)
	 * </p>
	 * <p>
	 * OrtsRefAsb: (BisAbschnitt == leer) || (BisAbschnitt==VonAbschnitt &&
	 * BisStation==VonStation)
	 * </p>
	 * 
	 * @return <code>true</code> wenn punktuelle Ortsreferenz, sonst
	 *         <code>false</code>
	 */
	private boolean isPunkt() {
		if (data.getArray("OrtsRefAlertC").getLength() > 0) {
			final Array array = data.getArray("OrtsRefAlertC");
			final Data item = array.getItem(0);

			return ((item.getItem("BisLCLPunkt").asUnscaledValue().longValue() == 0) || ((item.getItem("VonLCLPunkt")
					.asUnscaledValue().longValue() == item.getItem("BisLCLPunkt").asUnscaledValue().longValue())
					&& (item.getItem("VonLCLOffset").asUnscaledValue().longValue() == item.getItem("BisLCLOffset")
							.asUnscaledValue().longValue())));
		}

		if (data.getArray("OrtsRefAsb").getLength() > 0) {
			final Array array = data.getArray("OrtsRefAsb");
			final Data item = array.getItem(0);
			final String bisAbschnitt = item.getItem("BisAbschnitt").asTextValue().getText();
			final String vonAbschnitt = item.getItem("VonAbschnitt").asTextValue().getText();

			return bisAbschnitt.isEmpty() || (vonAbschnitt.equals(bisAbschnitt) && (item.getItem("VonStation")
					.asUnscaledValue().floatValue() == item.getItem("BisStation").asUnscaledValue().floatValue()));
		}

		if (data.getArray("OrtsRefAlertC").getLength() > 0) {
			final Array array = data.getArray("OrtsRefAlertC");
			final Data item = array.getItem(0);

			return ((item.getItem("BisLCLPunkt").asUnscaledValue().longValue() == 0) || ((item.getItem("VonLCLPunkt")
					.asUnscaledValue().longValue() == item.getItem("BisLCLPunkt").asUnscaledValue().longValue())
					&& (item.getItem("VonLCLOffset").asUnscaledValue().longValue() == item.getItem("BisLCLOffset")
							.asUnscaledValue().longValue())));
		}

		return false;
	}

	/**
	 * Gibt die Ortsreferenz als {@link GroupOfLocations} zur&uuml;ck.
	 * 
	 * @return {@link GroupOfLocations}
	 */
	public GroupOfLocations getOrtsReferenzen() {
		return groupOfLocations;
	}

	/**
	 * Setzt die Information, ob es in beiden Fahrtrichtungen Bauabschnitte
	 * gibt.
	 * 
	 * @param beideRichtungen
	 *            <code>true</code> wenn in beiden Fahrtrichtungen
	 *            Bauabschnitte, sonst <code>false</code>
	 */
	public void setBeideRichtungen(final boolean beideRichtungen) {
		if (groupOfLocations instanceof Linear) {
			((Datex2OrtsreferenzenLinear) groupOfLocations).setBeideRichtungen(beideRichtungen);
		}

	}
}
