/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ResultData;
import eu.datex2.schema.dynamisch.baustellen.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.baustellen.HeaderInformation;
import eu.datex2.schema.dynamisch.baustellen.InformationStatusEnum;
import eu.datex2.schema.dynamisch.baustellen.ObjectFactory;
import eu.datex2.schema.dynamisch.baustellen.Situation;
import eu.datex2.schema.dynamisch.baustellen.SituationExtended;
import eu.datex2.schema.dynamisch.baustellen.SituationExtensionType;
import eu.datex2.schema.dynamisch.baustellen.SituationRecord;

/**
 * Daten Datex2-Situation (Situation).
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public class Datex2Situation {

	private ResultData resultData;
	private ObjectFactory objectFactory;

	/**
	 * ERzeugt ein neues Objekt.
	 * 
	 * @param resultData
	 *            {@link ResultData}
	 * @param objectFactory
	 *            {@link ObjectFactory}
	 */
	public Datex2Situation(final ResultData resultData, final ObjectFactory objectFactory) {
		this.resultData = resultData;
		this.objectFactory = objectFactory;
	}

	/**
	 * Gibt das DATEX-II Situationsobjekt zur&uuml;ck.
	 * 
	 * @return {@link Situation}
	 * @throws Datex2BaustelleException
	 *             bei Fehlern
	 */
	public Situation getSituation() throws Datex2BaustelleException {
		final Situation sitation = objectFactory.createSituation();
		sitation.setHeaderInformation(buildHeaderInformation());
		sitation.setId(buildID());
		sitation.setVersion(buildVersion());

		// erst alle Bauabschnitte (SituationRecord)
		final List<SituationRecord> situationRecords = new ArrayList<>();

		final Array array = resultData.getData().getArray("Bauabschnitt");
		for (int i = 0; i < array.getLength(); i++) {
			situationRecords
					.add((new Datex2SituationRecordBauabschnitt(resultData, array.getItem(i))).getSituationRecord());
		}
		sitation.getSituationRecord().addAll(situationRecords);

		// Gesamtmassnahme
		final SituationExtensionType situationExtensionType = objectFactory.createSituationExtensionType();
		final SituationExtended situationExtended = objectFactory.createSituationExtended();

		situationExtended.setOverallSituation(
				(new Datex2SituationRecordGesamtMassnahme(resultData, resultData.getData().getItem("GesamtMassnahme")))
						.getSituationRecord());

		situationExtensionType.setSituationExtended(situationExtended);

		sitation.setSituationExtension(situationExtensionType);

		return sitation;
	}

	private HeaderInformation buildHeaderInformation() {
		final HeaderInformation headerInformation = objectFactory.createHeaderInformation();
		headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
		headerInformation.setInformationStatus(InformationStatusEnum.REAL);
		return headerInformation;
	}

	/**
	 * Bildet die ID der Situation.
	 * 
	 * @return ID der Situation
	 */
	private String buildID() {
		return resultData.getObject().getPid();
	}

	private String buildVersion() {
		return "" + getResultData().getDataTime();
	}

	/**
	 * Gibt zugeordnete {@link ResultData} zur&uuml;ck.
	 * 
	 * @return {@link ResultData}
	 */
	public ResultData getResultData() {
		return resultData;
	}
}
