/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente;

import java.util.List;

import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Datex2BaustelleException;
import eu.datex2.schema.dynamisch.baustellen.AffectedCarriagewayAndLanes;
import eu.datex2.schema.dynamisch.baustellen.CarriagewayEnum;

/**
 * Umsetzung Datex2 Sperrungsinfomationen in Datk
 * <code>att.mdmBisArtDerMaßnahmeKnotenZusatz</code>.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public enum Datex2ArtDerMassnahmeKnotenZusatzEnum {

	/**
	 * Kein Knoten&nbsp;0&nbsp;Baustelle liegt nicht in einem Knoten sondern auf
	 * der Strecke.
	 */
	KeinKnoten("Kein Knoten", 0),

	/**
	 * Einfahrt&nbsp;1&nbsp;Baustelle liegt in Anschlussstelle, Einfahrt
	 * betroffen.
	 */
	Einfahrt("Einfahrt", 1),

	/**
	 * Ausfahrt&nbsp;2&nbsp;Baustelle liegt in Anschlussstelle, Ausfahrt
	 * betroffen.
	 */
	Ausfahrt("Ausfahrt", 2),

	/**
	 * Ein-und Ausfahrt&nbsp;3&nbsp;Baustelle liegt in Anschlussstelle, Ein- und
	 * Ausfahrt betroffen.
	 */
	EinUndAusfahrt("Ein- und Ausfahrt", 3),

	/**
	 * Verbindungsfahrbahn&nbsp;4&nbsp;Baustelle liegt im Knoten,
	 * Verbindungsfahrbahn betroffen.
	 */
	Verbindungsfahrbahn("Verbindungsfahrbahn", 4),

	/**
	 * Parallelfahrbahn&nbsp;5&nbsp;Baustelle liegt im Knoten, Parallelfahrbahn
	 * betroffen.
	 */
	Parallelfahrbahn("Parallelfahrbahn", 5),

	/**
	 * Verbindungs-und Parallelfahrbahn&nbsp;6&nbsp;Baustelle liegt im Knoten,
	 * Verbindungsfahrbahn und Parallelfahrbahn betroffen.
	 */
	VerbindungsUndParallelfahrbahn("Verbindungs-und Parallelfahrbahn", 6);

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param carriageways
	 *            Liste von {@link CarriagewayEnum}
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static boolean isSperrungAnschlussStelle(final List<CarriagewayEnum> carriageways)
			throws Datex2BaustelleException {
		final boolean as = carriageways.contains(CarriagewayEnum.EXIT_SLIP_ROAD)
				|| carriageways.contains(CarriagewayEnum.ENTRY_SLIP_ROAD)
				|| carriageways.contains(CarriagewayEnum.SLIP_ROADS);

		// Teste Konsistenz (siehe Kap. 5.6.2: Unterscheidung Sperrung
		// Anschlussstelle/Knotenpunkt)
		if (as && (carriageways.contains(CarriagewayEnum.CONNECTING_CARRIAGEWAY)
				|| carriageways.contains(CarriagewayEnum.PARALLEL_CARRIAGEWAY))) {
			throw new Datex2BaustelleException(
					"Inkonsistente Angabe bei 'AffectedCarriagewayAndLanes': Die Liste enthält sowohl Kennungen für Anschlussstellen als auch für Knotenpunkte.");
		}

		return as;
	}

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param davwert
	 *            DAV-Wert
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2ArtDerMassnahmeKnotenZusatzEnum getInstance(final int davwert) throws Datex2BaustelleException {
		for (Datex2ArtDerMassnahmeKnotenZusatzEnum adm : Datex2ArtDerMassnahmeKnotenZusatzEnum.values()) {
			if (adm.getDavId() == davwert) {
				return adm;
			}
		}
		throw new Datex2BaustelleException(
				"Keine Datex2-'Datex2ArtDerMassnahmeKnotenZusatzEnum' zu DAV-Wert=" + davwert + " definiert.");
	}

	/**
	 * Gibt eine passende Instanz zur&uuml;ck.
	 *
	 * @param affectedCarriagewayAndLanes
	 *            Liste von {@link CarriagewayEnum}
	 * @return zum numerischen Wert passende Instanz
	 * @throws Datex2BaustelleException
	 *             bei ung&uuml;ltigem Wert
	 */
	public static Datex2ArtDerMassnahmeKnotenZusatzEnum getInstance(
			final List<AffectedCarriagewayAndLanes> affectedCarriagewayAndLanes) throws Datex2BaustelleException {

		boolean einfahrt = false;
		boolean ausfahrt = false;
		boolean verbindungsfahrbahn = false;
		boolean parallelfahrbahn = false;

		for (AffectedCarriagewayAndLanes carriagewaylane : affectedCarriagewayAndLanes) {
			switch (carriagewaylane.getCarriageway()) {
			case EXIT_SLIP_ROAD:
				ausfahrt = true;
				break;
			case ENTRY_SLIP_ROAD:
				einfahrt = true;
				break;
			case SLIP_ROADS:
				einfahrt = true;
				ausfahrt = true;
				break;
			case CONNECTING_CARRIAGEWAY:
				verbindungsfahrbahn = true;
				break;
			case PARALLEL_CARRIAGEWAY:
				parallelfahrbahn = true;
				break;

			default: // die anderen Typen sind im MDM-Datenmodell nicht
						// dokumentiert.
				break;
			}
		}

		// Teste Konsistenz (siehe Kap. 5.6.2: Unterscheidung Sperrung
		// Anschlussstelle/Knotenpunkt)
		if ((einfahrt || ausfahrt) && (verbindungsfahrbahn || parallelfahrbahn)) {
			throw new Datex2BaustelleException(
					"Inkonsistente Angabe bei 'AffectedCarriagewayAndLanes': Die Liste enthält sowohl Kennungen für Anschlussstellen als auch für Knotenpunkte.");
		}

		if (einfahrt && ausfahrt) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.EinUndAusfahrt;
		}

		if (einfahrt) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.Einfahrt;
		}

		if (ausfahrt) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.Ausfahrt;
		}

		if (verbindungsfahrbahn && parallelfahrbahn) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.VerbindungsUndParallelfahrbahn;
		}

		if (verbindungsfahrbahn) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.Verbindungsfahrbahn;
		}

		if (parallelfahrbahn) {
			return Datex2ArtDerMassnahmeKnotenZusatzEnum.Parallelfahrbahn;
		}

		throw new Datex2BaustelleException("'AffectedCarriagewayAndLanes' kann nicht ausgewertet werden:"
				+ affectedCarriagewayAndLanes.toString());
	}

	/**
	 * textueller Wert.
	 */
	private final String value;

	/**
	 * numerischer Wert.
	 */
	private int id;

	/**
	 * Privater Konstruktor.
	 *
	 * @param value
	 *            textueller Wert
	 * @param id
	 *            numerischer Wert
	 */
	Datex2ArtDerMassnahmeKnotenZusatzEnum(final String value, final int id) {
		this.value = value;
		this.id = id;
	}

	/**
	 * Gibt den numerischen Wert zur&uuml;ck.
	 *
	 * @return numerischer Wert der Himmelsrichtung
	 */
	public int getDavId() {
		return id;
	}

	/**
	 * Gibt den textuellen Wert zur&uuml;ck.
	 *
	 * @return textueller Wert der Himmelsrichtung
	 */
	public String getValue() {
		return value;
	}
}
