/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.publizierung.impl;

import java.io.ByteArrayOutputStream;
import java.io.NotActiveException;
import java.io.StringReader;
import java.net.URL;
import java.util.Date;

import javax.activation.UnsupportedDataTypeException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;
import javax.xml.ws.soap.SOAPBinding;

import org.xml.sax.SAXException;

import de.bsvrz.kex.datex2.plugins.definition.KExD2Plugin;
import de.bsvrz.kex.datex2.plugins.publizierung.PublizierungsMethode;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Übertragung einer XML-Struktur zum MDM.
 *
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class MDMPublizierung implements PublizierungsMethode {
  /** Standardwert-Timeout für Verbindungsaufbau (in Millisekunden). */
  private static final int DEFAULT_TIMEOUT = 15000;
  
  /** Timeout für Verbindungsaufbau (in Millisekunden). */
  private int timeout = DEFAULT_TIMEOUT;
  
  /** Zeichencodierung. */
  private static final String CHAR_ENCODING = "UTF-8";

  /** Namensraum für SupplierPush Service. */
  private static final String SERVICE_NAMESPACE = "http://datex2.eu/wsdl/supplierPush/2_0";
  
  /** Service-Name für SupplierPush. */
  private static final String SERVICE_NAME = "supplierPushService";
  
  /** Port-Name für SupplierPush. */
  private static final String PORT_NAME = "supplierPushSoapEndPoint";

  /** SOAP-Action URI für SupplierPush. */
  private static final String SOAP_ACTION_URI = "http://datex2.eu/wsdl/supplierPush/2_0/putDatex2Data";

  /** Datex II Namensraum. */
  private static final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";
  
  /** Root-Tag des Datex II Formats. */
  private static final String DATEX2_ROOTTAG = "d2LogicalModel";

  /** subContext gibt an, ob dynamische oder statische Daten übertragen werden. */
  private String subContext = "";
  
  /** Angabe der SchemaLocation im XML. */
  private String schemaLocation;
  
  /** Schema zur Validierung der auszugebenden XML-Struktur. */
  private Schema schema;

  /** Ziel des Soap-Aufrufs. */
  private String endpointURL;
  
  /** Referenz auf Plugin-Definition-Klasse. */
  private KExD2Plugin plugin;
  
  /** Präfix für Logging-Ausgaben. */
  private String logPrefix = "";
  
  /** Logger. */
  private static final Debug LOGGER = Debug.getLogger();

  /**
   * Konstruktor. Das übergebene Objekt vom Typ KEx2Plugin ist bei Instantiierung
   * noch nicht vollständig initialisiert.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse
   * @param subContext Angabe statisch oder dynamisch
   */
  
  public MDMPublizierung(final KExD2Plugin plugin, final String subContext) {
    this.plugin = plugin;
    this.subContext = subContext;
  }

  /** {@inheritDoc} */
  @Override
  public boolean initialisiere() {
    boolean hasAllData = true;
    this.logPrefix = "In Kontext '" + plugin.getKonfigurationsModul().getContext() + "': ";
    schemaLocation = plugin.getKonfigurationsModul().getString("SchemaLocation-" + subContext);
    String schemaFilename = plugin.getKonfigurationsModul().getString("SchemaDatei-" + subContext);

    // Auf dem MDM darf keine Ausgabe ohne SchemaLocation erfolgen
    
    if (schemaLocation == null) {
      hasAllData = false;
    }
    
    try {
      URL schemaFile = new URL(schemaFilename);
      SchemaFactory schemaFactory = SchemaFactory
          .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
      schema = schemaFactory.newSchema(schemaFile);
    } catch (Exception ex) {
      LOGGER.warning(logPrefix + "Schema-Validierung konnte nicht initialisiert werden, "
          + "Überprüfung bei Publizierung findet nicht statt.", ex);
    }
    
    try {
      endpointURL = plugin.getKonfigurationsModul().getData("MDM-URL-" + subContext, String.class);
    } catch (NotActiveException | UnsupportedDataTypeException
        | IllegalArgumentException e) {
      LOGGER.error(logPrefix + "DateiName-" + subContext + " konnte nicht ausgewertet werden.");
      hasAllData = false;
    }
    
    try {
      timeout = plugin.getKonfigurationsModul().getData("MDM-Connection-Timeout-" + subContext, Integer.class);
    } catch (NotActiveException | UnsupportedDataTypeException
        | IllegalArgumentException e) {
      LOGGER.fine(logPrefix + " (" + subContext + ") Nutze Standard-Connection-Timeout.");
    }

    return hasAllData;
  }
  
  /** {@inheritDoc} */
  @Override
  public String getPublizierungsMethodenName() {
    return "MDM";
  }

  /**
   * 
   * @param src Zugriff auf (marshalled) XML-Struktur
   * @return true, wenn
   *             a) kein Schema initialisiert wurde
   *             b) XML ist konform zu Schema
   *         false, wenn
   *             a) ein allgemeiner Fehler während der Validierung auftritt
   *             b) XML nicht konform zu Schema ist.
   */
  protected boolean validiere(final Source src) {
    if (schema == null) {
      return true;
    } else {
      try {
        Validator validator = schema.newValidator();
    
        validator.validate(src);
        return true;
      } catch (SAXException e) {
        LOGGER.error(logPrefix + "Validierungsfehler, MDM-Publikation wird abgebrochen. ", e);
      } catch (Exception ex) {
        LOGGER.error(logPrefix + "Fehler bei Validierung, MDM-Publikation wird abgebrochen. ", ex);        
      }
    }
    return false;
  }

//} catch (Exception ex) {
//LOGGER.error(logPrefix + "Datenübertragung zum MDM konnte nicht durchgeführt werden.", ex);
//plugin.getErweiterterLogger().sendeAnOperator(logPrefix 
//    + "Datenübertragung zum MDM konnte nicht durchgeführt werden " + ex.getMessage());
//}
//try {

  /** {@inheritDoc} 
   * 
   * Führe keine Publizierung durch, wenn destination leer ist oder Validierung nicht erfolgreich.
   */
  @Override
  public void publiziere(final Object xml, final JAXBContext jaxbContext, final String param) throws Exception {
    if (!endpointURL.isEmpty()) {
      LOGGER.fine(logPrefix + "Bereite MDM-Publizierung vor.");
      QName qualifiedServiceName = new QName(SERVICE_NAMESPACE, SERVICE_NAME);

      Service serv = Service.create(qualifiedServiceName);
      QName qualifiedPortName = new QName(SERVICE_NAMESPACE, PORT_NAME);

      serv.addPort(qualifiedPortName, SOAPBinding.SOAP11HTTP_BINDING, endpointURL);

      ByteArrayOutputStream ba = new ByteArrayOutputStream();

      Marshaller marshaller = jaxbContext.createMarshaller();
      marshaller.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, schemaLocation);
      marshaller.setProperty(Marshaller.JAXB_ENCODING, CHAR_ENCODING);
      marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE, DATEX2_ROOTTAG), Object.class, xml), ba);

      Source xmls = new StreamSource(new StringReader(ba.toString(CHAR_ENCODING)));

      if (validiere(xmls)) {
        Source xmls2 = new StreamSource(new StringReader(ba.toString(CHAR_ENCODING)));
        Dispatch<Source> dispatch = serv.createDispatch(qualifiedPortName, Source.class, Service.Mode.PAYLOAD);

        dispatch.getRequestContext().put(BindingProvider.SOAPACTION_USE_PROPERTY, Boolean.TRUE);
        dispatch.getRequestContext().put(BindingProvider.SOAPACTION_URI_PROPERTY, SOAP_ACTION_URI);

        // Die folgenden Festlegungen könnten in naher Zukunft um korrigierte Einträge
        // ergänzt werden müssen.
        dispatch.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", timeout);
        dispatch.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", timeout);
        dispatch.getRequestContext().put("com.sun.xml.ws.request.timeout", timeout);
        dispatch.getRequestContext().put("com.sun.xml.ws.connect.timeout", timeout);

        LOGGER.info(logPrefix + "Starte MDM-Publizierung nach " + endpointURL);
        Date beforeSubmit = new Date();

        Source result = dispatch.invoke(xmls2);

        // Die Rückgabe wird nicht weiter untersucht (keine Anforderung diesbezüglich).
        // Es wird nur geprüft, ob überhaupt eine Antwort gekommen ist.

        if (result == null) {
          throw new Exception("Keine Antwort vom Server erhalten.");
        }

        Date afterSubmit = new Date();
        long duration = afterSubmit.getTime() - beforeSubmit.getTime();
        LOGGER.info(logPrefix + "MDM-Publizierung abgeschlossen (" + ba.size() + " Bytes in " + duration + " ms)");
      } else {
        throw new Exception("Validierung fehlerhaft.");
      }
    }
  }
}